<?php
/**
 * @copyright CometChat
 * @author support@cometchat.com
 */

// make sure we are not being called directly
defined('APP_DIR') or exit();

/**
 * meta
 */
function ccCometChat_meta()
{
    $_tmp = array(
        'name'        => 'CometChat',
        'url'         => 'www.cometchat.com',
        'version'     => '1.0.1',
        'developer'   => 'CometChat, &copy;2015',
        'description' => 'Add CometChat to your Jamroom site',
        'category'    => 'communication',
        'license'     => 'mpl'
    );
    return $_tmp;
}

/**
 * init
 */
function ccCometChat_init()
{
    jrCore_register_event_listener('jrCore', 'view_results', 'ccCometChat_view_results_listener');
    jrCore_register_event_listener('jrCore', 'form_validate_init', 'ccCometChat_form_validate_init_listener');
    jrCore_register_event_listener('jrCore', 'verify_module', 'ccCometChat_verify_module_listener');
    jrCore_register_module_feature('jrCore', 'admin_tab', 'ccCometChat', 'admin_install', 'Install');
    jrCore_register_module_feature('jrCore', 'admin_tab', 'ccCometChat', 'admin_panel', 'Administration Panel');

    // Core module support
    $_tmp = array(
        'label'   => 'Allow CometChat',
        'help'    => 'If checked, profiles in this quota will be allowed to use CometChat',
        'default' => 'on'
    );
    jrCore_register_module_feature('jrCore', 'quota_support', 'ccCometChat', 'on', $_tmp);

    return true;
}

//---------------------------------------------------------
// EVENT LISTENERS
//---------------------------------------------------------

/**
 * add two lines of HTML code to the site footer
 * @param $_data array incoming data array
 * @param $_user array current user info
 * @param $_conf array Global config
 * @param $_args array additional info about the module
 * @param $event string Event Trigger name
 * @return array
 */
function ccCometChat_view_results_listener($_data, $_user, $_conf, $_args, $event)
{
    global $_post;

    if (isset($_post['__ajax']) && $_post['__ajax'] == '1') {
        return $_data;
    }
    if ((!isset($_post['_uri']) || !strpos($_post['_uri'], 'admin')) && isset($_conf['ccCometChat_active']) && $_conf['ccCometChat_active'] == 'on') {

        $show_cometchat = true;
        if ($_user['quota_ccCometChat_allowed'] != 'on') {
            $show_cometchat = false;
        }

        $baseurl = 'modules/ccCometChat/contrib/cometchat';

        $footer_code = '';
        if ($show_cometchat) {
            $footer_code = '<link type="text/css" href="' . $_conf['jrCore_base_url'] . '/' . $baseurl . '/css.php" rel="stylesheet" charset="utf-8" /><script type="text/javascript" src="' . $_conf['jrCore_base_url'] . '/' . $baseurl . '/js.php" charset="utf-8"></script>';
        }
        return str_replace('</body>', $footer_code . '</body>', $_data);
    }
    return $_data;
}

function ccCometChat_form_validate_init_listener($_data, $_user, $_conf, $_args, $event)
{
    if (isset($_data['option']) && isset($_data['module']) && $_data['option'] == 'admin_save' && $_data['module'] == 'ccCometChat') {
        $initbaseUrl = $_conf['ccCometChat_baseurl'];
        $baseUrl     = $_data['baseurl'];
        if ($initbaseUrl != $baseUrl) {
            $base    = $_conf['jrCore_base_url'] . '/' . $baseUrl . '/';
            $file    = $_conf['jrCore_base_dir'] . '/' . $baseUrl . '/config.php';
            $content = @file_get_contents($file);

            if ($content != '') {

                $myvar = "define('BASE_URL','{$base}');";

                $pattern = "/\/\* BASE URL START \*\/(\s*)(.*?)(\s*)\/\* BASE URL END \*\//is";

                $replacement = "/* BASE URL START */\r\n\r\n" . $myvar . "\r\n\r\n/* BASE URL END */";

                $content = preg_replace($pattern, $replacement, $content);

                $f = @fopen($file, 'w');
                if ($f) {
                    @fwrite($f, $content);
                    @fclose($f);
                }
            }
            if ($handle = opendir($_conf['jrCore_base_dir'] . '/' . $baseUrl . '/cache/')) {
                while (false !== ($file = readdir($handle))) {
                    if ($file != "." && $file != ".." && $file != "index.html") {
                        @unlink($_conf['jrCore_base_dir'] . '/' . $baseUrl . '/cache/' . $file);
                    }
                }
            }
        }
    }
    return $_data;
}


/**
 * Fired when the integrity check runs.
 * @param $_data array Array of information from trigger
 * @param $_user array Current user
 * @param $_conf array Global Config
 * @param $_args array additional parameters passed in by trigger caller
 * @param $event string Triggered Event name
 * @return array
 */
function ccCometChat_verify_module_listener($_data, $_user, $_conf, $_args, $event)
{

    // make sure the users table is up-to-date
    $tbl = jrCore_db_table_name('ccCometChat', 'cometchat_jamroom_users');
    $req = "insert ignore into {$tbl}(userid,username,user_group)(SELECT item_id_tbl._item_id userid, username_tbl.value username, user_group_tbl.value user_group from jr_jruser_item_key item_id_tbl join jr_jruser_item_key username_tbl on item_id_tbl._item_id = username_tbl._item_id and username_tbl.key = 'user_name' join jr_jruser_item_key user_active_tbl on item_id_tbl._item_id = user_active_tbl._item_id and user_active_tbl.key = 'user_active' and user_active_tbl.value = '1' join jr_jruser_item_key user_group_tbl on item_id_tbl._item_id = user_group_tbl._item_id and user_group_tbl.key = 'user_group'  group by item_id_tbl._item_id);";
    jrCore_db_query($req);

    return $_data;
}